/*
** CCTDE - Covert Channels and Tunnels Detection Engine - v0.2
** Copyright (C) 2003,2004 Simon Castro - scastro@entreelibre.com
**
** This file is part of CCTDE - v0.2 (C) 2003,2004 Simon Castro.
** 
** Cctde is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctde is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctde; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
** These functions belong to and are updated from the Gray-world CCTT project.
*/

#include "../includes/cctde.h"

t_conf_struct conf_upcon[]= {
  { "uptime", CF_MAX_UPCON_UPTIME },
  { "prx", CF_MAX_UPCON_PRX },
  { "ptx", CF_MAX_UPCON_PTX },
  { "brx", CF_MAX_UPCON_BRX },
  { "btx", CF_MAX_UPCON_BTX },
  { NULL, 0 }
};

t_conf_struct conf_max[]= {
  { "con", CF_MAX_CON },
  { "uptime", CF_MAX_UPTIME },
  { "prx", CF_MAX_PRX },
  { "ptx", CF_MAX_PTX },
  { "brx", CF_MAX_BRX },
  { "btx", CF_MAX_BTX },
  { "srv", CF_MAX_SRV },
  { "alert", CF_MAX_ALERT },
  { NULL, 0 }
};

t_conf_struct conf_top[]= {
  { "con", CF_MAX_CON },
  { "uptime", CF_MAX_UPTIME },
  { "prx", CF_MAX_PRX },
  { "ptx", CF_MAX_PTX },
  { "brx", CF_MAX_BRX },
  { "btx", CF_MAX_BTX },
  { "srv", CF_MAX_SRV },
  { "alert", CF_MAX_ALERT },
  { NULL, 0 }
};

/*
** Configuration functions
*/

int conf_file_SNORT_LABEL(char *function, char *params, int flag)
{
  if (!(G_cctde_infos.config.snort_label = (char *) my_malloc(strlen(params)+1)))
    {
      display_infos(stderr,"conf_file_SNORT_LABEL ... Malloc error\n");
      exit (-1);
    }
  memset(G_cctde_infos.config.snort_label,0,strlen(params)+1);
  memcpy(G_cctde_infos.config.snort_label,params,strlen(params));
  return (1);
}

int conf_file_TIMELINE_PERIOD(char *function, char *params, int flag)
{
  int tmp=0;

  if (flag == 0) {
    if ((tmp = atoi(params)) < 1) return (0);
    G_cctde_infos.config.timeline_period = tmp;
    return (1);
  }
  if (flag == 1) {
    if ((tmp = atoi(params)) < 0) return (0);
    G_cctde_infos.config.timeline_nb = tmp;
    return (1);
  }
  if (flag == 2) {
    if ((tmp = atoi(params)) < 0 || tmp > 3) return (0);
    G_cctde_infos.config.timeline_flush = tmp;
    return (1);
  }
  return (0);
}

int conf_file_FILES(char *function, char *params, int flag)
{
  time_t tmp_time;
  char *time_now;

  if (flag == 0) {
    if (!(G_cctde_infos.config.cmd_file = (char *) my_malloc(strlen(params)+1))) {
      display_infos(stderr,"conf_file_FILES ... Malloc error\n");
      exit (-1);
    }
    memset(G_cctde_infos.config.cmd_file,0,strlen(params)+1);
    memcpy(G_cctde_infos.config.cmd_file,params,strlen(params));
  }
  if (flag == 1) {
    tmp_time = current_time(0);
    time_now = get_str2_from_time_t(tmp_time);
    if (!(G_cctde_infos.config.bk_file = (char *) my_malloc(strlen(params)+strlen(time_now)+2))) {
      display_infos(stderr,"conf_file_FILES ... Malloc error\n");
      exit (-1);
    }
    memset(G_cctde_infos.config.bk_file,0,strlen(params)+strlen(time_now)+2);
    memcpy(G_cctde_infos.config.bk_file,params,strlen(params));
    G_cctde_infos.config.bk_file[strlen(params)] = '_';
    memcpy(G_cctde_infos.config.bk_file+strlen(params)+1,time_now,strlen(time_now));
  }
  return (1);
}

int conf_file_DISPLAY(char *function, char *params, int flag)
{
  int tmp=0;

  if (flag == 0) {
    if ((tmp = atoi(params)) < 0 || tmp > 2) return (0);
    G_cctde_infos.config.disp_tcp = tmp;
    return (1);
  }
  if (flag == 1) {
    if ((tmp = atoi(params)) < 0 || tmp > 1) return (0);
    G_cctde_infos.config.disp_cs = tmp;
    return (1);
  }
  if (flag == 2) {
    if ((tmp = atoi(params)) < 0 || tmp > 1) return (0);
    G_cctde_infos.config.disp_sm = tmp;
    return (1);
  }
  if (flag == 3) {
    if ((tmp = atoi(params)) < 0 || tmp > 1) return (0);
    G_cctde_infos.config.disp_tn = tmp;
    return (1);
  }
  if (flag == 4) {
    if ((tmp = atoi(params)) < 0 || tmp > 1) return (0);
    G_cctde_infos.config.disp_tnc = tmp;
    return (1);
  }
  if (flag == 5) {
    if ((tmp = atoi(params)) < 0 || tmp > 1) return (0);
    G_cctde_infos.config.disp_res = tmp;
    return (1);
  }
  if (flag == 6) {
    if ((tmp = atoi(params)) < 0 || tmp > 1) return (0);
    G_cctde_infos.config.disp_cur = tmp;
    return (1);
  }
  if (flag == 7) {
    if ((tmp = atoi(params)) < 0 || tmp > 2) return (0);
    G_cctde_infos.config.disp_cli = tmp;
    return (1);
  }
  if (flag == 8) {
    if ((tmp = atoi(params)) < 0 || tmp > 1) return (0);
    G_cctde_infos.config.disp_sri = tmp;
    return (1);
  }
  if (flag == 9) {
    if ((tmp = atoi(params)) < 0 || tmp > 1) return (0);
    G_cctde_infos.config.disp_topx = tmp;
    return (1);
  }
  if (flag == 10) {
    if ((tmp = atoi(params)) < 0 || tmp > 1) return (0);
    G_cctde_infos.config.disp_arules = tmp;
    return (1);
  }
  return (0);
}

int conf_file_MAX_UPCON(char *function, char *params, int flag)
{
  int i;
  static char type[MAX_LABEL];
  static char val[MAX_VAL];

  for(i=0 ; params[i] && params[i] != ';' && i < MAX_LABEL; i++)
    ;
  if (i > MAX_LABEL-1 || strlen(params) > i+MAX_VAL) return (0);
  
  memset(type,0,MAX_LABEL);
  memset(val,0,MAX_VAL);
  strncpy(type,params,i);
  strncpy(val,params+i+1,strlen(params)-i);
  
  for (i=0;conf_upcon[i].label != NULL;i++)
    if (strlen(conf_upcon[i].label) == strlen(type) &&
	strncmp(conf_upcon[i].label,type,strlen(type)) == 0) {
      G_cctde_infos.config.conf_upcon[conf_upcon[i].pos] = (u_int32_t) atoi(val);
      return (1);
    }
  return (0);
}

int match_alert_NUM(t_correlator_alert *stored_alert, t_correlator_alert *new_alert)
{
  if (stored_alert->NUM == new_alert->NUM)
    return(1);
  return(-1);
}

int conf_file_ALERT_MAX(char *function, char *params, int flag)
{
  int i,j,k,l;
  static char label[MAX_LABEL],NUM[MAX_LABEL];
  static char max[MAX_VAL],dal[MAX_VAL],pal[MAX_VAL];
  static char silent[MAX_VAL],weight[MAX_VAL];
  t_correlator_alert alert;
  t_correlator_alert_max alert_max;
  
  memset(NUM,0,MAX_LABEL);
  memset(label,0,MAX_LABEL);
  memset(max,0,MAX_VAL);
  memset(dal,0,MAX_VAL);
  memset(pal,0,MAX_VAL);
  memset(silent,0,MAX_VAL);
  memset(weight,0,MAX_VAL);
  memset(&alert,0,sizeof(t_correlator_alert));
  memset(&alert_max,0,sizeof(t_correlator_alert_max));

  for (i=0,j=1,k=0,l=0;i<strlen(params)||k<7;i++,j++)
    {
      if (params[i] == ';' || i == strlen(params))
	{
	  if ((k < 2 && j > MAX_LABEL-1) || (k > 1 && j > MAX_VAL-1))
	    return (0);
	  if (j-1 < 1) return (0);
	  if (k == 6) { strncpy(weight,params+l,j-1); k+=1; }
	  if (k == 5) { strncpy(silent,params+l,j-1); k+=1; }
	  if (k == 4) { strncpy(pal,params+l,j-1); k+=1; }
	  if (k == 3) { strncpy(dal,params+l,j-1); k+=1; }
	  if (k == 2) { strncpy(max,params+l,j-1); k+=1; }
	  if (k == 1) { strncpy(label,params+l,j-1); k+=1; }
	  if (k == 0) { strncpy(NUM,params+l,j-1); k+=1; }
	  j=0; l=i+1;
	}
    }
  if (k != 7)
    return (0);

  if ((alert.NUM = atoi(NUM)) < 1 || alert.NUM > MAX_ALERT_NUM)
    {
      display_infos(stderr,"Cannot use NUM %d! please use 0 < NUM <= %d.\n",
		    alert.NUM,MAX_ALERT_NUM);
      return (0);
    }
  alert.alert_type = ALERT_MAX;
  if (flag == 0) 
    alert.cl_or_srv = ALERT_CL;
  else
    alert.cl_or_srv = ALERT_SRV;

  for (i=0;conf_max[i].label != NULL;i++)
    {
      if (strlen(conf_max[i].label) == strlen(label) &&
	  strncmp(conf_max[i].label,label,strlen(label)) == 0)
	alert_max.label = conf_max[i].pos;
    }

  if (alert_max.label == 0)
    return (0);
  alert_max.max = atoi(max);
  alert_max.dal = atoi(dal);
  alert_max.pal = atoi(pal);
  alert_max.silent = atoi(silent);
  alert_max.weight = atoi(weight);
  memcpy(&alert.alert_max,&alert_max,sizeof(t_correlator_alert_max));

  if ((look_for_elem(&(G_cctde_infos.config.correlator_alerts),&alert,match_alert_NUM)) != NULL)
    {
      display_infos(stderr,"NUM already exists.\n");
      return (0);
    }
  
  if (add_to_list(&(G_cctde_infos.config.correlator_alerts),
		  &alert,sizeof(t_correlator_alert),1) == NULL)
    {
      display_infos(stderr,"Couldn't add alert to correlator alert list.\n");
      return (0);
    }
  return (1);

}

int conf_file_ALERT_ADV(char *function, char *params, int flag)
{
  int i,j,k,l;
  static char cmp1[MAX_LABEL],cmp2[MAX_LABEL],NUM[MAX_LABEL];
  static char nbcmp1[MAX_VAL],mul[MAX_VAL],res[MAX_VAL];
  static char dal[MAX_VAL],pal[MAX_VAL],silent[MAX_VAL],weight[MAX_VAL];
  static t_correlator_alert alert;
  static t_correlator_alert_adv alert_adv;

  memset(NUM,0,MAX_LABEL);
  memset(cmp1,0,MAX_LABEL);
  memset(cmp2,0,MAX_LABEL);
  memset(nbcmp1,0,MAX_VAL);
  memset(mul,0,MAX_VAL);
  memset(res,0,MAX_VAL);
  memset(dal,0,MAX_VAL);
  memset(pal,0,MAX_VAL);
  memset(silent,0,MAX_VAL);
  memset(weight,0,MAX_VAL);
  memset(&alert,0,sizeof(t_correlator_alert));
  memset(&alert_adv,0,sizeof(t_correlator_alert_adv));

  for (i=0,j=1,k=0,l=0;i<strlen(params)||k<10;i++,j++)
    {
      if (params[i] == ';' || i == strlen(params))
	{
	  if ((k < 3 && j > MAX_LABEL-1) || (k > 2 && j > MAX_VAL-1))
	    return (0);
	  if (j-1 < 1) return (0);
	  if (k == 9) { strncpy(weight,params+l,j-1); k+=1; }
	  if (k == 8) { strncpy(silent,params+l,j-1); k+=1; }
	  if (k == 7) { strncpy(pal,params+l,j-1); k+=1; }
	  if (k == 6) { strncpy(dal,params+l,j-1); k+=1; }
	  if (k == 5) { strncpy(res,params+l,j-1); k+=1; }
	  if (k == 4) { strncpy(mul,params+l,j-1); k+=1; }
	  if (k == 3) { strncpy(nbcmp1,params+l,j-1); k+=1; }
	  if (k == 2) { strncpy(cmp2,params+l,j-1); k+=1; }
	  if (k == 1) { strncpy(cmp1,params+l,j-1); k+=1; }
	  if (k == 0) { strncpy(NUM,params+l,j-1); k+=1; }
	  j=0; l=i+1;
	}
    }
  if (k != 10)
    return (0);

  if ((alert.NUM = atoi(NUM)) < 1 || alert.NUM > MAX_ALERT_NUM)
    {
      display_infos(stderr,"Cannot use NUM %d! please use 0 < NUM <= %d.\n",
		    alert.NUM,MAX_ALERT_NUM);
      return (0);
    }
  alert.alert_type = ALERT_ADV;
  if (flag == 0) 
    alert.cl_or_srv = ALERT_CL;
  else
    alert.cl_or_srv = ALERT_SRV;

  for (i=0;conf_max[i].label != NULL;i++)
    {
      if (strlen(conf_max[i].label) == strlen(cmp1) &&
	  strncmp(conf_max[i].label,cmp1,strlen(cmp1)) == 0)
	alert_adv.cmp1 = conf_max[i].pos;
      if (strlen(conf_max[i].label) == strlen(cmp2) &&
	  strncmp(conf_max[i].label,cmp2,strlen(cmp2)) == 0)
	alert_adv.cmp2 = conf_max[i].pos;
    }
  
  if (alert_adv.cmp1 == 0 || alert_adv.cmp2 == 0)
    return (0);
  alert_adv.nbcmp1 = atoi(nbcmp1);
  alert_adv.mul = atoi(mul);
  alert_adv.res = atoi(res);
  alert_adv.dal = atoi(dal);
  alert_adv.pal = atoi(pal);
  alert_adv.silent = atoi(silent);
  alert_adv.weight = atoi(weight);
  memcpy(&alert.alert_adv,&alert_adv,sizeof(t_correlator_alert_adv));

  if ((look_for_elem(&(G_cctde_infos.config.correlator_alerts),&alert,match_alert_NUM)) != NULL)
    {
      display_infos(stderr,"NUM already exists.\n");
      return (0);
    }
  
  if (add_to_list(&(G_cctde_infos.config.correlator_alerts),
		  &alert,sizeof(t_correlator_alert),1) == NULL)
    {
      display_infos(stderr,"Couldn't add alert to correlator alert list.\n");
      return (0);
    }
  return (1);
}

int conf_file_TOP(char *function, char *params, int flag)
{
  int i;
  static char type[MAX_LABEL];
  static char val[MAX_VAL];

  for(i=0 ; params[i] && params[i] != ';' && i < MAX_LABEL; i++)
    ;
  if (i > MAX_LABEL-1 || strlen(params) > i+MAX_VAL)
    return (0);

  memset(type,0,MAX_LABEL);
  memset(val,0,MAX_VAL);
  strncpy(type,params,i);
  strncpy(val,params+i+1,strlen(params)-i);

  for (i=0;conf_top[i].label != NULL;i++)
    if (strlen(conf_top[i].label) == strlen(type) &&
	strncmp(conf_top[i].label,type,strlen(type)) == 0)
      {
	if (flag == 0)
	  G_cctde_infos.config.conf_topX_cl[conf_top[i].pos] = (u_int32_t) atoi(val);
	else
	  G_cctde_infos.config.conf_topX_srv[conf_top[i].pos] = (u_int32_t) atoi(val);
	return (1);
      }
  return (0);
}

/*
**
*/

typedef struct s_brws_cfile {
  char *label;
  int (*fn)();
  int flag;
} t_brws_cfile;

static t_brws_cfile brws_cfile[] = {
  { "A_SNORT_LABEL" , conf_file_SNORT_LABEL , 0 },

  { "A_TIMELINE_PERIOD", conf_file_TIMELINE_PERIOD , 0 },
  { "A_KEEP_TIMELINE", conf_file_TIMELINE_PERIOD , 1 },
  { "A_FLUSH_TIMELINE", conf_file_TIMELINE_PERIOD , 2 },

  { "A_DISPLAY_TCP_CONNECTION", conf_file_DISPLAY , 0 },
  { "A_DISPLAY_COLD_START", conf_file_DISPLAY , 1 },
  { "A_DISPLAY_SNORT_MESSAGES", conf_file_DISPLAY , 2 },
  { "A_DISPLAY_TIMELINE", conf_file_DISPLAY , 3 },
  { "A_DISPLAY_TIMELINE_NO_CLIENT", conf_file_DISPLAY , 4 },
  { "A_DISPLAY_RESUME", conf_file_DISPLAY , 5 },
  { "A_DISPLAY_CURRENT", conf_file_DISPLAY , 6 },
  { "A_DISPLAY_CLIENTS_INFOS", conf_file_DISPLAY , 7 },
  { "A_DISPLAY_SERVERS_INFOS", conf_file_DISPLAY , 8 },
  { "A_DISPLAY_TOPX", conf_file_DISPLAY , 9 },
  { "A_DISPLAY_ALERT_RULES", conf_file_DISPLAY , 10 },

  { "A_MAX_UPCON", conf_file_MAX_UPCON , 0 },

  { "A_TOP_X_CL", conf_file_TOP , 0 },
  { "A_TOP_X_SRV", conf_file_TOP , 1 },

  { "A_COMMAND_FILE", conf_file_FILES , 0 },

  { "A_BACKUP_FILE", conf_file_FILES , 1 },

  { "ALERT_MAX_CL", conf_file_ALERT_MAX, 0 },
  { "ALERT_MAX_SRV", conf_file_ALERT_MAX, 1},

  { "ALERT_ADV_CL", conf_file_ALERT_ADV, 0 },
  { "ALERT_ADV_SRV", conf_file_ALERT_ADV, 1 },
  
  { NULL, NULL , 0 }
};

/*
** Looks on the ptr function array if "function" exists and
** executes it...
*/

int search_config_function(char *function,char *params)
{
  int i;

  for (i=0;brws_cfile[i].label != NULL;i++)
    if (strncmp(brws_cfile[i].label,function,strlen(function)) == 0)
      {
	if (!params || strlen(params) < 1)
	  {
	    display_infos(stderr,"In %s, %s is not valid...\n",
		     G_cctde_infos.config.conf_file,
		     function);
	    return (0);
	  }
	if ((brws_cfile[i].fn(function,params,brws_cfile[i].flag)) == 0)
	  {
	    display_infos(stderr,"In %s, %s=%s is not valid...\n",
		     G_cctde_infos.config.conf_file,
		     function,params);
	    return (0);
	  }
	else
	  return (1);
      }
  display_infos(stderr,"In %s, %s is not valid...\n",
	   G_cctde_infos.config.conf_file,function);
  return (0);
}

/*
** Browse the config file and for each line, call the function
** which will search the related function ptr...
*/

int  parse_conf_file(int (browse_conf)(), char *filename)
{
  int  fd_conf_file,nb_oct_conf_file;
  char *conf_file, *conf_file_free;
  char *ptr,*function,*params;
  char test;

  /* Peut-on ouvrir le fichier de configuration ? */ 
  fd_conf_file = open(filename,O_RDONLY);
  if (fd_conf_file < 1)
    {
      display_infos(stderr,"Cannot open %s configuration file !\n",filename);
      return (-1);
    }

  /* Nombre d'octet du fichier de configuration + 1 */
  nb_oct_conf_file = lseek(fd_conf_file,0,SEEK_END);
  if (nb_oct_conf_file < 1)
    {
      close(fd_conf_file);
      display_infos(stderr,"Configuration file is empty !\n");
      return (-1);
    }

  /* Fait attention a la derniere ligne du fichier (si elle est finie par un '\n' ou non */
  lseek(fd_conf_file,-1,SEEK_END);
  if ((read(fd_conf_file,&test,1) == 1) && test != '\n')
    nb_oct_conf_file += 1;

  /* Recuperation du contenu du fichier de configuration */
  conf_file = (char *) my_malloc (nb_oct_conf_file);
  memset (conf_file,0,nb_oct_conf_file);
  lseek(fd_conf_file,0,SEEK_SET);
  read(fd_conf_file,conf_file,nb_oct_conf_file-1);

  /* If test == 1 after the while, it means we had no error in cfile */
  test = 0;

  /* Sauvegarde du pointeur pour le free et
     parsage du buffer ligne par ligne */
  conf_file_free = conf_file;
  while ((ptr = strsep(&conf_file,"\n")))
    {
      /* Si la ligne n'est pas vide et qu'il n'y a pas de caractere
	 indiquand un commentaire */
      if (strlen(ptr) > 1 && !index(ptr,'#'))
	{
	  function = strsep(&ptr,"=");
	  params = strsep(&ptr,"=");
	  if ((browse_conf(function,params)) == 0)
	    test = 1;
	}
    }

  my_free (conf_file_free,strlen(conf_file_free),"conf_file.c / parse_conf_file");
  close(fd_conf_file);

  if (test == 1)
    return (-1);
  return (1);
}
