/*
** CCTDE - Covert Channels and Tunnels Detection Engine - v0.2
** Copyright (C) 2003,2004 Simon Castro - scastro@entreelibre.com
**
** This file is part of CCTDE - v0.2 (C) 2003,2004 Simon Castro.
** 
** Cctde is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctde is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctde; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/cctde.h"

int correlate_upcon(t_con_track *ptr)
{
  upcon_detect_uptime(ptr);
  upcon_detect_packets_amount(ptr);
  upcon_detect_bytes_amount(ptr);
  return (1);
}

int generic_stats_client(t_correlator_alert *stored_alert, t_storage *stored_view)
{
  if (stored_alert->cl_or_srv != ALERT_CL)
    return (0);

  if (stored_alert->alert_type == ALERT_ADV)
    advanced_alerts(stored_alert,stored_view,"Client");
  if (stored_alert->alert_type == ALERT_MAX)
    max_alerts(stored_alert,stored_view,"Client");

  return (0);
}

int generic_stats_server(t_correlator_alert *stored_alert, t_storage *stored_view)
{
  if (stored_alert->cl_or_srv != ALERT_SRV)
    return (0);

  if (stored_alert->alert_type == ALERT_ADV)
    advanced_alerts(stored_alert,stored_view,"Server");
  if (stored_alert->alert_type == ALERT_MAX)
    max_alerts(stored_alert,stored_view,"Server");

  return (0);
}

int correlate_client(t_storage *stored_view, void *rien)
{
  look_for_elem(&(G_cctde_infos.config.correlator_alerts),stored_view,generic_stats_client);
  return (0); // browse the whole list
}

int correlate_server(t_storage *stored_view, void *rien)
{
  look_for_elem(&(G_cctde_infos.config.correlator_alerts),stored_view,generic_stats_server);
  return (0); // browse the whole list
}

int correlate_and_alert()
{
  int i;
  t_con_track *ptr;

  /* 
  ** First doing detection on current connections :
  **   $ Connection lasting for more than x seconds;
  **   $ More than x rx and/or tx packets on the same connection;
  **   $ More than x rx and/or tx bytes on the same connection;
  */
  for(i=0; i<CON_HASH_SIZE; i++) 
    if (G_con_track[i] != NULL)
      for (ptr = G_con_track[i]; ptr != NULL; ptr = ptr->next)
	correlate_upcon(ptr);

  /*
  ** Now doing connections on stored informations :
  **   $ We parse each list (client or server) once.
  **   $ For each element, we parse the alert list once.
  **   $ For each (element,alert), two kind of correlation are done :
  **     * basic correlation based on A_MAX_X directives.
  **     * advanced correlation based on A_ADV_X directives.
  */
  
  look_for_elem(&G_cctde_infos.client_list,(void *) NULL,correlate_client);
  look_for_elem(&G_cctde_infos.server_list,(void *) NULL,correlate_server);

  return (1);
}

void display_alerts_infos(t_correlator_alert *stored_alert)
{
  display_infos(stdout,"    # Alert #%d for %s (%s : ",
		stored_alert->NUM,
		(stored_alert->cl_or_srv == ALERT_SRV ? "servers" : "clients"),
		(stored_alert->alert_type == ALERT_MAX ? "Max" : "Advanced"));

  if (stored_alert->alert_type == ALERT_ADV) {
    if (stored_alert->alert_adv.mul != 0)
      display_infos(stdout,"%s (> %d) && %s > %s * %d ?)\n      dal: %d, pal: %d - silent for %d alerts, weight: %d\n",
		    conf_max[stored_alert->alert_adv.cmp1].label,
		    stored_alert->alert_adv.nbcmp1,
		    conf_max[stored_alert->alert_adv.cmp1].label,
		    conf_max[stored_alert->alert_adv.cmp2].label,
		    stored_alert->alert_adv.mul,
		    stored_alert->alert_adv.dal,stored_alert->alert_adv.pal,
		    stored_alert->alert_adv.silent,stored_alert->alert_adv.weight);
    if (stored_alert->alert_adv.res != 0)
      display_infos(stdout,"%s (> %d) && %s / %s > %d ?)\n      dal: %d, pal: %d, - silent for %d alerts, weight: %d\n",
		    conf_max[stored_alert->alert_adv.cmp1].label,
		    stored_alert->alert_adv.nbcmp1,
		    conf_max[stored_alert->alert_adv.cmp1].label,
		    conf_max[stored_alert->alert_adv.cmp2].label,
		    stored_alert->alert_adv.res,
		    stored_alert->alert_adv.dal,stored_alert->alert_adv.pal,
		    stored_alert->alert_adv.silent,stored_alert->alert_adv.weight);
    
  }
  
  if (stored_alert->alert_type == ALERT_MAX) {
    display_infos(stdout,"%s > %d ?)\n      dal: %d, pal: %d - silent for %d alerts, weight: %d\n",
		  conf_max[stored_alert->alert_max.label].label,
		  stored_alert->alert_max.max,
		  stored_alert->alert_max.dal,stored_alert->alert_max.pal,
		  stored_alert->alert_max.silent,stored_alert->alert_max.weight);
  }
}

int display_alert_rules(char *function, char *params, int flag)
{
  display_infos(stdout,"  o %d alert rule(s)\n",
		G_cctde_infos.config.correlator_alerts.elem_nb);
  browse_existing_elems(&(G_cctde_infos.config.correlator_alerts),display_alerts_infos);
  return (1);
}

int remove_alert_rules(char *function, char *params, int flag)
{
  t_list_elem *elem;
  static t_correlator_alert alert;

  if (!params || strlen(params) < 1)
    return (0);
  if ((alert.NUM = atoi(params)) < 1 || alert.NUM > MAX_ALERT_NUM)
    return (0);
  
  if ((elem = look_for_elem(&(G_cctde_infos.config.correlator_alerts),
			    &alert,match_alert_NUM)) != NULL)
    {
      if (remove_elem(&(G_cctde_infos.config.correlator_alerts),elem,1) < 1)
	{
	  display_infos(stdout,"  Couldn't remove Alert rule #%d from memory.\n",alert.NUM);
	  return (0);
	}
      else
	display_infos(stdout,"  Alert rule #%d removed.\n",alert.NUM);

    }
  else
    display_infos(stdout,"  Alert rule #%d doesn't exist.\n",alert.NUM);
  return (1);
}
