/*
** CCTDE - Covert Channels and Tunnels Detection Engine - v0.2
** Copyright (C) 2003,2004 Simon Castro - scastro@entreelibre.com
**
** This file is part of CCTDE - v0.2 (C) 2003,2004 Simon Castro.
** 
** Cctde is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctde is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctde; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/cctde.h"

typedef struct s_advanced_infos {
  u_int32_t type;
  char *label;
  u_int32_t value;
} t_advanced_infos;

int advanced_alerts(t_correlator_alert *stored_alert, t_storage *stored_view, char *cl_or_srv)
{
  t_advanced_infos advanced_infos[] = {
    { CF_MAX_CON , "con", stored_view->tcp_nb },
    { CF_MAX_UPTIME , "uptime", stored_view->com_uptime },
    { CF_MAX_PRX , "prx", stored_view->packet_rx },
    { CF_MAX_PTX , "ptx", stored_view->packet_tx },
    { CF_MAX_BRX , "brx", stored_view->bytes_rx },
    { CF_MAX_BTX , "btx", stored_view->bytes_tx },
    { CF_MAX_SRV , "srv", stored_view->sec_list_qt },
    { CF_MAX_ALERT , "alert", stored_view->total_alerts },
    { 0 , NULL , 0 }
  };
  int NUM,ret;
  u_int32_t cmp1,nbcmp1,cmp2,mul,res,dal,pal,silent;

  cmp1 = stored_alert->alert_adv.cmp1;
  cmp2 = stored_alert->alert_adv.cmp2;
  nbcmp1 = stored_alert->alert_adv.nbcmp1;
  mul = stored_alert->alert_adv.mul;
  res = stored_alert->alert_adv.res;
  dal = stored_alert->alert_adv.dal;
  pal = stored_alert->alert_adv.pal;
  silent = stored_alert->alert_adv.silent;
  NUM = stored_alert->NUM;
  
  if (mul != 0)
    {
      if (advanced_infos[cmp1].value > nbcmp1 && 
	  advanced_infos[cmp1].value > advanced_infos[cmp2].value * mul)
	{
	  if ((ret = alert_dal_and_pal(stored_view,NUM,advanced_infos[cmp1].value,
				       dal,pal,silent)) >= 0)
	    {
	      display_infos(stdout,"[%s] CCTDE - %s Alert #%d (Advanced) : ",
			    get_str_from_time_t(current_time(1)),cl_or_srv,NUM);
	      display_infos(stdout,"%s %s(%d) (not before %d) > %s(%d)*%d",
			    ip_uint_to_str(stored_view->ip),
			    advanced_infos[cmp1].label,advanced_infos[cmp1].value,nbcmp1,
			    advanced_infos[cmp2].label,advanced_infos[cmp2].value,mul);
	      if (ret <= 0) display_infos(stdout,".\n");
	      if (ret == 1) display_infos(stdout," (rem. %d secs).\n",dal);
	      if (ret == 2) display_infos(stdout," (rem. %d%%).\n",pal);
	    }
	}
    }
  
  if (res != 0)
    {
      if (advanced_infos[cmp1].value < 1 || advanced_infos[cmp2].value < 1)
	return (0); // Break now !
      if (advanced_infos[cmp1].value > nbcmp1 && 
	  advanced_infos[cmp1].value / advanced_infos[cmp2].value > res)
	{
	  if ((ret = alert_dal_and_pal(stored_view,NUM,advanced_infos[cmp1].value,
				       dal,pal,silent)) >= 0)
	    {
	      display_infos(stdout,"[%s] CCTDE - %s Alert #%d (Advanced) : ",
			    get_str_from_time_t(current_time(1)),cl_or_srv,NUM);
	      display_infos(stdout,"%s %s(%d) / %s(%d) > %d",ip_uint_to_str(stored_view->ip),
			    advanced_infos[cmp1].label,advanced_infos[cmp1].value,
			    advanced_infos[cmp2].label,advanced_infos[cmp2].value,res);
	      if (ret <= 0) display_infos(stdout,".\n");
	      if (ret == 1) display_infos(stdout," (rem. %d secs).\n",dal);
	      if (ret == 2) display_infos(stdout," (rem. %d%%).\n",pal);
	    }
	}
    }
  
  /* Take care to return 0 to browse the whole list */
  return (0);
}
