/*
** CCTDE - Covert Channels and Tunnels Detection Engine - v0.2
** Copyright (C) 2003,2004 Simon Castro - scastro@entreelibre.com
**
** This file is part of CCTDE - v0.2 (C) 2003,2004 Simon Castro.
** 
** Cctde is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctde is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctde; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/cctde.h"

typedef struct s_max_infos {
  u_int32_t pos;
  char *label;
  u_int32_t value;
  char *alert_str;
} t_max_infos;

int alert_dal_and_pal(t_storage *stored_view, u_int32_t NUM, u_int32_t check_val, u_int32_t reminder_dal, u_int32_t reminder_pal, u_int32_t silent)
{
  u_int32_t aval=0,atime=0;
  
  aval = stored_view->alert_stats[NUM].alert_value;
  atime = stored_view->alert_stats[NUM].last_alert;
  if (aval == 0 || atime == 0) // First alert
    {
      stored_view->total_alerts += 1;
      stored_view->alert_stats[NUM].alert_value = check_val;
      stored_view->alert_stats[NUM].last_alert = current_time(1);
      stored_view->alert_stats[NUM].alerts_number += 1;
      if (silent == 0 || stored_view->alert_stats[NUM].alerts_number > silent)
	return (0);
      else
	return (-1);
    }
  if (reminder_dal != 0 && (current_time(1) >= (atime + reminder_dal)))
    {
      stored_view->total_alerts += 1;
      stored_view->alert_stats[NUM].last_alert = current_time(1);
      stored_view->alert_stats[NUM].alerts_number += 1;
      if (silent == 0 || stored_view->alert_stats[NUM].alerts_number > silent)
	return(1);
      else
	return (-1);
    }
  if (reminder_pal != 0 && 
      ((((check_val - aval) * 100) / aval) >= reminder_pal))
    {
      stored_view->total_alerts += 1;
      stored_view->alert_stats[NUM].alert_value = check_val;
      stored_view->alert_stats[NUM].alerts_number += 1;
      if (silent == 0 || stored_view->alert_stats[NUM].alerts_number > silent)
	return(2);
      else
	return (-1);
    }
  return (-1);
}

int max_alerts(t_correlator_alert *stored_alert, t_storage *stored_view, char *cl_or_srv)
{
  int ret,NUM;
  u_int32_t label,max,dal,pal,silent;
  t_max_infos max_infos[] = {
    { CF_MAX_CON , "con", stored_view->tcp_nb , 
      "Conn. reached %d : %s openned %d connections"},
    { CF_MAX_UPTIME , "uptime", stored_view->com_uptime ,
      "Global uptime reached %d secs : %s stood up for %d secs" },
    { CF_MAX_PRX , "prx", stored_view->packet_rx ,
      "Prx reached %d : %s received %d packets" },
    { CF_MAX_PTX , "ptx", stored_view->packet_tx ,
      "Ptx reached %d : %s sent %d packets"},
    { CF_MAX_BRX , "brx", stored_view->bytes_rx ,
      "Brx reached %d : %s received %d bytes" },
    { CF_MAX_BTX , "btx", stored_view->bytes_tx , 
      "Btx reached %d : %s sent %d bytes" },
    { CF_MAX_SRV , "srv", stored_view->sec_list_qt ,
      "srv reached %d : %s connected to %d servers" },
    { CF_MAX_ALERT , "alert", stored_view->total_alerts ,
      "alert reached %d : %s generated %d alerts" },
    { 0 , NULL , 0 , NULL }
  };
  
  label = stored_alert->alert_max.label;
  max = stored_alert->alert_max.max;
  dal = stored_alert->alert_max.dal;
  pal = stored_alert->alert_max.pal;
  silent = stored_alert->alert_max.silent;
  NUM = stored_alert->NUM;
  
  if (max != 0 && max_infos[label].value >= max)
    if ((ret = alert_dal_and_pal(stored_view,NUM,max_infos[label].value,dal,pal,silent)) >= 0)
      {
	display_infos(stdout,"[%s] CCTDE - %s Alert #%d (Max) : ",
		      get_str_from_time_t(current_time(1)),cl_or_srv,NUM);
	display_infos(stdout,max_infos[label].alert_str,max,
		      ip_uint_to_str(stored_view->ip),max_infos[label].value);
	if (ret <= 0) display_infos(stdout,".\n");
	if (ret == 1) display_infos(stdout," (rem. %d secs).\n",dal);
	if (ret == 2) display_infos(stdout," (rem. %d%%).\n",pal);
      }
  /* Take care to return 0 to browse the whole list */
  return (0);
}
