/*
** CCTDE - Covert Channels and Tunnels Detection Engine - v0.2
** Copyright (C) 2003,2004 Simon Castro - scastro@entreelibre.com
**
** This file is part of CCTDE - v0.2 (C) 2003,2004 Simon Castro.
** 
** Cctde is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctde is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctde; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/cctde.h"

int is_it_locked(t_storage *stored_view, void *rien)
{
  if (stored_view->lock == 1)
    return (1);
  return (-1);
}

void remove_servers_from_clist(t_storage *client_infos)
{
  if (free_that_list(&client_infos->second_list) != 1)
    display_infos(stderr,"Problem while freeing server_list from time period...\n");
}

#define TLINE_FLUSH 0
#define TLINE_DISP  1
#define TLINE_WRITE 2
#define TLINE_DISP_WR 3

int timeline_is_locked(t_time_storage *time_storage)
{
  char logfile[MAX_LOGFILE_SIZE];
  char *time_now;
  FILE *timeline_file;

  t_list_elem *force;
  int i;
  t_con_track *ptr;

  if (time_storage->client_list.elem_nb != 0 &&
      look_for_elem(&(time_storage->client_list),NULL, is_it_locked) != NULL)
    {
      // Only uncomment for debug purpose...
      //      display_infos(stderr,"  o Time list created the %s is currently locked.\n",
      //	    get_str_from_time_t(time_storage->time_creation));
      return (1);
    }
  else
    {
      if (G_cctde_infos.config.timeline_flush == TLINE_DISP ||
	  G_cctde_infos.config.timeline_flush == TLINE_DISP_WR)
	display_timeline(time_storage);

      if (G_cctde_infos.config.timeline_flush == TLINE_WRITE ||
	  G_cctde_infos.config.timeline_flush == TLINE_DISP_WR)
	{

	  time_now = get_str2_from_time_t(time_storage->time_creation);
	  memset(logfile,0,MAX_LOGFILE_SIZE);
	  if (G_cctde_infos.config.logfile)
	    {
	      memcpy(logfile,G_cctde_infos.config.logfile,strlen(G_cctde_infos.config.logfile));
	      logfile[strlen(G_cctde_infos.config.logfile)] = '_';
	      memcpy(logfile+strlen(G_cctde_infos.config.logfile)+1,time_now,strlen(time_now));
	    }
	  else
	    {
	      memcpy(logfile,DEFAULT_TIMELINE_LOGFILE,strlen(DEFAULT_TIMELINE_LOGFILE));
	      logfile[strlen(DEFAULT_TIMELINE_LOGFILE)] = '_';
	      memcpy(logfile+strlen(DEFAULT_TIMELINE_LOGFILE)+1,time_now,strlen(time_now));
	    }

	  if (!(timeline_file = get_stream_from_file(logfile,1))) {
	    display_infos(stderr,"Cannot open %s to backup timeline!\n",logfile);
	    return (-1);
	  } else {
	    backup_timeline(time_storage,timeline_file);
	    fclose(timeline_file);
	  }
	}
      
      if (time_storage->client_list.elem_nb != 0)
	{
	  for (force = time_storage->client_list.first;force;force = force->next)
	    {
	      for(i=0; i<CON_HASH_SIZE; i++) 
		if (G_con_track[i] != NULL)
		  for (ptr = G_con_track[i]; ptr != NULL; ptr = ptr->next)
		    {
		      if (ptr->infos == force)
			{
			  ptr->infos = NULL;
			  ptr->infos2 = NULL;
			  ptr->infos3 = NULL;
			}
		    }
	    }
	  browse_existing_elems(&(time_storage->client_list),remove_servers_from_clist);
	  if (free_that_list(&time_storage->client_list) != 1)
	    display_infos(stderr,"Problem while freeing client_list from time period...\n");
	}
    }
  return (0);
}

int manage_timelines()
{
  t_list_elem *tmp,*tmp2;
  int i,num;
  
  // Keep all timelines in memory
  if (CONSIDER_TIMELINE() == 0 || 
      (num = G_cctde_infos.config.timeline_nb) == 0)
    return (0);

  if (G_cctde_infos.time_list.elem_nb > num)
    {
      tmp = G_cctde_infos.time_list.first;
      for (i=0;i<num-1&&tmp;i++)
	tmp = tmp->next;
      tmp2 = G_cctde_infos.time_list.last;

      while (tmp2 && memcmp(tmp,tmp2,sizeof(t_list_elem)) != 0)
	{
	  // Timeline locked means open connections are recording
	  // in this timeline at this time...
	  if (tmp2->data && timeline_is_locked(tmp2->data) != 1)
	    {
	      // We remove the time element because servers and clients were
	      // removed in the timeline_is_locked function...
	      if ((tmp2 = tmp2->prev))
		{
		  if (remove_elem(&(G_cctde_infos.time_list),tmp2->next,1) < 1)
		    display_infos(stderr,"Problem while removing timeline !!!\n");
		}
	      else
		{
		  if (remove_elem(&(G_cctde_infos.time_list),tmp2,1) < 1)
		    display_infos(stderr,"Problem while removing timeline !!!\n");
		  tmp2 = NULL;
		}
	    }
	  else
	    tmp2 = tmp2->prev;
	}
    }
  return (0);
}
